/**
 * @file
 * @author  Caleb Fangmeier <caleb@fangmeier.tech>
 * @version 0.1
 *
 * @section LICENSE
 *
 *
 * MIT License
 *
 * Copyright (c) 2017 Caleb Fangmeier
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef config_hpp
#define config_hpp
#include <string>

#include "yaml-cpp/yaml.h"

namespace fv::util {
struct DataFileDescriptor{
    std::string filename;
    std::string label;
    std::string category;
    int file_number;

    DataFileDescriptor():filename(""), label(""),category(""),file_number(-1) { }

    DataFileDescriptor(const std::string filename, const std::string label, const std::string category)
      :filename(filename), label(label),category(category),file_number(-1) { }

    DataFileDescriptor(const std::string filename, const std::string label, const std::string category, const int file_number)
      :filename(filename), label(label),category(category),file_number(file_number) { }
};
}

namespace YAML {
template<>
struct convert<fv::util::DataFileDescriptor> {
  static Node encode(const fv::util::DataFileDescriptor& rhs) {
    Node node;
    node["filename"] = rhs.filename;
    node["label"] = rhs.label;
    node["category"] = rhs.category;
    return node;
  }

  static bool decode(const Node& node, fv::util::DataFileDescriptor& rhs) {
    if(!node.IsMap()) {
      return false;
    }

    if(!node["filename"]) return false;
    rhs.filename = node["filename"].as<std::string>();
    if(node["label"]) rhs.label = node["label"].as<std::string>();
    if(node["category"]) rhs.category = node["category"].as<std::string>();
    return true;
  }
};
}

namespace fv::util {
class Config{
    private:
        std::string input_filename;
        YAML::Node root;
    public:
        Config ()
          :root(), input_filename("") { }
        Config (const Config& c)
          :root(c.root), input_filename(c.input_filename) { }
        Config (const std::string& input_filename)
          :root(YAML::LoadFile(input_filename)),
           input_filename(input_filename) { }

        template <typename KEY_TYPE>
        YAML::Node get(const KEY_TYPE& key) {
            auto val = root[key];
            return val;
        }

        size_t get_max_events() {
            YAML::Node max_events = root["max-events"];
            if(!max_events) {
                return 0;
            }
            return max_events.as<size_t>();
        }

        std::string get_output_filename() {
            YAML::Node output_file = root["output-file"];
            if(output_file) {
                return output_file.as<std::string>();
            } else {
                std::cout << "Must specify output-file in config file." << std::endl;
                return "";
            }
        }

        std::vector<DataFileDescriptor> get_source_files() {
            std::vector<DataFileDescriptor> source_files;
            YAML::Node source_files_nd = root["source-files"];
            if(source_files_nd.IsSequence()) {
                for (const auto& f : source_files_nd) {
                    source_files.push_back(f.as<DataFileDescriptor>());
                }
            } else {
                std::cout << "source-files missing or not a sequence" << std::endl;
            }
            return source_files;
        }
};

Config* the_config = nullptr;
void init_config(const std::string& input_filename){
    if(the_config) delete the_config;
    the_config = new Config(input_filename);
}

}
#endif // config_hpp
